@@ -16,7 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <fcntl.h>
 #include <getopt.h>
@@ -38,14 +37,13 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <assert.h>
-#include <config.h>
 #include <urcu/compiler.h>
 #include <ulimit.h>
 
 #include <common/defaults.h>
 #include <common/common.h>
-#include <common/consumer.h>
-#include <common/consumer-timer.h>
+#include <common/consumer/consumer.h>
+#include <common/consumer/consumer-timer.h>
 #include <common/compat/poll.h>
 #include <common/compat/getenv.h>
 #include <common/sessiond-comm/sessiond-comm.h>
@@ -103,7 +101,7 @@ static void sighandler(int sig)
 
 	/*
 	 * Ignore SIGPIPE because it should not stop the consumer whenever a
-	 * SIGPIPE is catched through a FD operation.
+	 * SIGPIPE is caught through a FD operation.
 	 */
 	if (sig == SIGPIPE) {
 		return;
