@@ -3012,3 +3012,9 @@ end:
 	}
 	return ret;
 }
+
+static
+void __attribute__((destructor)) session_config_exit(void)
+{
+	xmlCleanupParser();
+}
