@@ -16,7 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h>
@@ -109,8 +108,7 @@ static void empty_session_list(void)
 	struct ltt_session *iter, *tmp;
 
 	cds_list_for_each_entry_safe(iter, tmp, &session_list->head, list) {
-		cds_list_del(&iter->list);
-		free(iter);
+		session_destroy(iter);
 	}
 
 	/* Session list must be 0 */
